import json

try:
    from django.core.urlresolvers import resolve
except:
    from django.urls import resolve
from django.forms import ChoiceField


class RelatedChoiceField(ChoiceField):
    related_url = None
    related_dependent = None
    empty_label = None

    def __init__(self, related_url=None, related_dependent=None, empty_label=None, **kwargs):
        self.related_url = related_url
        self.related_dependent = related_dependent
        self.empty_label = empty_label if empty_label is not None else '--------'
        super(RelatedChoiceField, self).__init__(**kwargs)

    def widget_attrs(self, widget):
        return {
            'data-related-url': self.related_url,
            'data-related-dependent': self.related_dependent,
            'data-empty-label': self.empty_label
        }

    def init_bound_field(self, obj, request_user=None):
        class FakeRequest(object):
            method = 'GET'
            GET = {'value': obj}
            user = request_user

        resolver_match = resolve(self.related_url)
        response = resolver_match.func(FakeRequest())
        choices = []
        content = response.content.decode()
        if not self.required:
            choices.append(('', self.empty_label))
        for item in json.loads(content):
            choices.append((item['value'], item['key']))
        self.choices = choices

    def validate(self, value):
        if self.choices:
            # actually checks vs. choices
            return super(RelatedChoiceField, self).validate(value)
        else:
            # does the normal check for blank and required=True
            return super(ChoiceField, self).validate(value)



from django.http import JsonResponse
from django.views.generic import View


class abstractstatic(staticmethod):
    __slots__ = ()

    def __init__(self, function):
        super(abstractstatic, self).__init__(function)
        function.__isabstractmethod__ = True

    __isabstractmethod__ = True


class RelatedSelectView(View):
    """
    AJAX GET view for related select form fields.  Expects a GET request to some URL similar to
    '/autocomplete-view/?value=123', where 123 will be passed into `filter()` to generate the list
    returned to populate the dependent select field
    """

    @abstractstatic
    def filter(value, **kwargs):
        return []

    @staticmethod
    def to_text(obj):
        return str(obj)

    @staticmethod
    def to_value(obj):
        return obj.id

    @staticmethod
    def blank_value():
        return None  # override to provide a blank value

    def dispatch(self, request, *args, **kwargs):
        if request.method != 'GET':
            raise NotImplementedError('This view only accepts GET requests')
        v = request.GET.get('value', None)
        ajax_list = []
        if self.blank_value():
            ajax_list.append(self.blank_value())
        for model_instance in self.filter(v, user=request.user):
            ajax_list.append({'key': self.to_text(model_instance),
                              'value': self.to_value(model_instance)})
        return JsonResponse(ajax_list, safe=False)



from setuptools import setup

setup(
    name='django-related-select',
    packages=['related_select'],
    version='0.9.3',
    description='Class-based View and django form field for related select boxes',
    author='Zach McCormick',
    author_email='zachary.tyler.mccormick@gmail.com',
    url='https://github.com/zachmccormick/django-related-select',
    keywords=['django', 'related', 'dependent', 'forms'],
    classifiers=[],
    install_requires=[
        'django'
    ],
    test_suite='tests',
    include_package_data=True
)
